package util

import (
	"samba/pkg/repo"
	"testing"
)

type userData struct {
	Sex  int
	Icon string
}
type user struct {
	Name string
	Age  int
	Data userData
}

func TestStructToMap(t *testing.T) {
	u := user{
		Name: "liu",
		Age:  1,
		Data: userData{
			Sex:  1,
			Icon: "zero",
		},
	}
	m, err := StructToMap(u)
	if err != nil {
		t.Error(err)
	}
	t.Log(m)
	u2 := &user{}
	err = MapToStruct(m, u2)
	if err != nil {
		t.Error(err)
	}
	t.Log(u2)
}

type info struct {
	Sex  int
	Nick string
}

type struct2redisTest struct {
	Id      int
	Name    string
	Players []int64
	Info    *info
	Books   map[string]string
}

func TestStruct2Redis(t *testing.T) {
	rdb, err := repo.InitRedis("", "testbuild.shoa.com", "6379", 0)
	if err != nil {
		t.Error(err)
	}
	value := struct2redisTest{
		Id:      100,
		Name:    "test",
		Players: []int64{1, 2, 3},
		Info: &info{
			Sex:  1,
			Nick: "nick",
		},
		Books: map[string]string{
			"100": "中国",
			"101": "美国",
		},
	}
	err = Struct2Redis(rdb, "testhello", value)
	if err != nil {
		t.Error(err)
	}
}

func TestRedis2Struct(t *testing.T) {
	rdb, err := repo.InitRedis("", "testbuild.shoa.com", "6379", 0)
	if err != nil {
		t.Error(err)
	}
	value, err := Redis2Struct[struct2redisTest](rdb, "testhello")
	if err != nil {
		t.Error(err)
	}
	kvs, err := Serialize(value)
	if err != nil {
		t.Error(err)
	}
	t.Logf("%#v", kvs)
}
